/******************************************************************************/
/* OpenSi : Outils libres de gestion d'entreprise                             */
/* Copyright (C) 2003 Speedinfo.fr S.A.R.L.                                   */
/* Contact: contact@opensi.org                                                */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/

/**
 * Classe d'origine par OpenSI
 * Modifiée par jm pour AEL Sarl
 * Données nécessaires à l' édition d'une facture standard
 */

package org.opensi.facturation.actions.documents.modeles;


import java.sql.Connection;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.TreeMap;

import org.experlog.openeas.api.Session;
import org.opensi.util.tools.Arrondi;
import org.opensi.util.tools.StringUtils;

/**
 * Classe originale OpenSI modifiée par jm pour AEL Sarl
 * Représente les données d'une facture
 */

public class DataFacture {


	private DecimalFormatSymbols dfs2 = new DecimalFormatSymbols();
	
	private DecimalFormat numFormatQte = new DecimalFormat("0.##");
	private SimpleDateFormat formatDate = new SimpleDateFormat("dd.MM.yy");

	// les lignes de la facture
	protected ArrayList<LigneFacture> lignes = new ArrayList<LigneFacture>(10);
	
	// coordonnées société émettrice

	protected String raisonSociale;
	protected String adresse;
	protected String compAdresse;
	protected String adresse3;
	protected String codePostal;
	protected String ville;
	protected String pays;
	protected String tel;
	protected String fax;
	protected String email;
	protected String web;
	
	protected double capital;
	protected String typeSociete;
	protected String tvaIntra;
	protected String rcs;
	protected String siren;
	
	// coordonnées du destinataire
	
	protected String clientId;
	protected String raisonSocialeDest;
	protected String adresseDest;
	protected String compAdresseDest;
	protected String adresseDest3;
	protected String codePostalDest;
	protected String villeDest;
	protected String paysDest;
	
	// coordonn�es de livraison
	
	protected String denominationLiv;
	protected String adresse1Liv;
	protected String adresse2Liv;
	protected String adresse3Liv;
	protected String codePostalLiv;
	protected String villeLiv;
	protected String paysLiv;
	
	protected String codePaysLiv;

	protected String numTvaClient = "";
	
	// infos facture
	
	protected String numeroFacture;
	protected String interlocuteur = "";
	protected String interlocuteurLiv = "";
	protected boolean zoneUE=false;
	protected String modeReglement;
	protected String typeReglement;
	protected String comFin = "";
	protected String mentions = "";
	protected String codeFournisseur;
	protected String refCommande;
	protected String refAffaire;
	protected String refBL;
	
	protected long dateFacture;
	protected long dateEcheance;
	
	protected double tRemise = 0;
	protected double tEscompte = 0;
	protected double mRemise = 0;
	protected double mEscompte = 0;
	protected double acompte = 0;
	protected double fraisPort = 0;
	protected double montantBase = 0;
	protected double totalBase = 0;
	protected double totalTVA = 0;
	protected double montantTTC = 0;
	protected double netAPayer = 0;
	protected double tauxTvaPort = 0;
	protected double ecoTaxe = 0;	
	
	protected double montantBaseInd;
	protected double mRemiseInd;
	protected double fraisPortInd = 0;
	protected double totalBaseInd = 0;
	protected double totalTVAInd = 0;
	protected double montantTTCInd = 0;
	protected double netAPayerInd = 0;
	protected double mEscompteInd = 0;
	protected double acompteInd = 0;
	
	protected double tauxInd = 0;

	protected boolean sansarticles = false;
	protected boolean editionTTC = false;
	protected boolean rist = false;
	protected boolean coul = false;
	protected boolean tail = false;
	protected boolean unite = false;
	protected boolean nbPieces = false;
	protected boolean numLot = false;
	protected boolean datePeremption = false;
	protected boolean logoAdr;
	protected String urlLogo = "";
	protected String urlApercu = "";
	
	protected String urlFlecheHautLCR = "";
	protected String urlFlecheBasLCR = "";
	protected String urlPetiteFlecheLCR = "";
	
	protected int nbex;
	protected int directe;
	protected boolean imprimerAdrLiv;
	protected boolean imprimerLCR;
	protected boolean apercu;
	
	protected String regimeTVA = "G";
	protected boolean exonerationTVA = false;
	
	private int comWebId;
	protected boolean assujetti;
	protected boolean TVALiv;
	protected String typePays;
	

	protected ArrayList<VentilTVA> vtvas = new ArrayList<VentilTVA>(2);
	protected ArrayList<VentilTVA> vtvasNpr = new ArrayList<VentilTVA>(2);
	
	private Arrondi r = new Arrondi(2);
	private String codePays;


	/**
	 * Constructeur appelé par défaut
	 */
	public DataFacture(Session s, String factureId) {
		this(s, factureId, false, 0);
	}
	
	
	public DataFacture(Session s, String factureId, boolean apercu, long dateFacture) {	
		try {		
			Connection con = s.getConnection(null);
			Statement stt = con.createStatement();
			Statement stt2 = con.createStatement();
			Statement stt3 = con.createStatement();
			
			String dossierId = s.getCookie().get("Dossier_Id");
			String base = s.getCookie().get("BaseDossier");
			
			this.urlLogo = s.getAppConfiguration().get("opensi.absdir") +"/logos/"+ dossierId +".jpg";
			this.urlApercu = s.getAppConfiguration().get("opensi.absdir") +"/images/apercu.jpg";
			this.urlFlecheHautLCR = s.getAppConfiguration().get("opensi.absdir") +"/images/fleche_haut.jpg";
			this.urlFlecheBasLCR = s.getAppConfiguration().get("opensi.absdir") +"/images/fleche_bas.jpg";
			this.urlPetiteFlecheLCR = s.getAppConfiguration().get("opensi.absdir") +"/images/petite_fleche.jpg";
			this.apercu = apercu;

			ResultSet rset;
			
			dfs2.setDecimalSeparator('.');
			numFormatQte.setDecimalFormatSymbols(dfs2);
			
      
			// coordonnées de l'entreprise facturante
      
			String reqFacturante = "select Ville_RCS, Num_TVA_Intra, Type_Societe, Montant_Capital, Num_SIRET, Raison_Sociale, Adresse, Comp_Adresse, Code_Postal, Ville, Telephone, Fax, Email, Site_Web from DOSSIER where Dossier_Id='"+ dossierId +"'";
			
			rset = stt.executeQuery(reqFacturante);
			
			rset.next();
			
			this.raisonSociale = rset.getString("Raison_Sociale");
			this.adresse = rset.getString("Adresse");
			this.compAdresse = rset.getString("Comp_Adresse");
			this.adresse3 = "";
			this.codePostal = rset.getString("Code_Postal");
			this.ville = rset.getString("Ville");
			this.tel = rset.getString("Telephone");
			this.fax = rset.getString("Fax");
			this.email = rset.getString("Email");
			this.web = rset.getString("Site_Web");
			
			this.siren = formatSiren(rset.getString("Num_SIRET"));
			this.typeSociete = rset.getString("Type_Societe");
			this.capital = rset.getDouble("Montant_Capital");
			this.tvaIntra = rset.getString("Num_TVA_Intra");
			this.rcs = rset.getString("Ville_RCS");		
			
			rset.close();
			
			// El�ments de la facture
			
			String reqFacture = "select f.*, c1.Civ_Courte as Civ_Courte_Fact, c2.Civ_Courte as Civ_Courte_Liv, p1.Nom_FR as Pays, p2.Nom_FR as Pays_Liv, p2.Zone_UE, p2.Type_Pays, r.Type_R from "+ base +".FACTURE f, CIVILITE c1, CIVILITE c2, PAYS p1, PAYS p2, "+ base +".MODE_REGLEMENT r";
			reqFacture += " where c1.Civ_Id=f.Civ_Inter and c2.Civ_Id=f.Civ_Inter_Liv and f.Code_Pays=p1.Code_Pays and f.Code_Pays_Liv=p2.Code_Pays and f.Mode_Reg=r.Mode_R and f.Facture_Id="+ factureId;
			
			rset = stt.executeQuery(reqFacture);
			
			rset.next();
			
			this.numeroFacture = rset.getString("Num_Entier");
			this.dateFacture = this.apercu ? dateFacture : rset.getLong("Date_Facture");
			this.tEscompte = rset.getDouble("Escompte");
			this.acompte = rset.getDouble("Acompte");
			this.tRemise = rset.getDouble("Remise");
			this.fraisPort = rset.getDouble("Frais_Port");
			this.dateEcheance = rset.getLong("Echeance");
			
			this.raisonSocialeDest = rset.getString("Raison_Sociale");
			this.adresseDest = rset.getString("Adresse_Fact");
			this.compAdresseDest = rset.getString("Comp_Adresse_Fact");
			this.adresseDest3 = rset.getString("Adresse_3_Fact");
			this.codePostalDest = rset.getString("CP_Fact");
			this.villeDest = rset.getString("Ville_Fact");
			this.paysDest = rset.getString("Pays");
			this.codePays = rset.getString("Code_Pays");
			this.denominationLiv = rset.getString("Denomination_Liv");
			this.adresse1Liv = rset.getString("Adresse_1_Liv");
			this.adresse2Liv = rset.getString("Adresse_2_Liv");
			this.adresse3Liv = rset.getString("Adresse_3_Liv");
			this.codePostalLiv = rset.getString("Code_Postal_Liv");
			this.villeLiv = rset.getString("Ville_Liv");
			this.paysLiv = rset.getString("Pays_Liv");
			this.codePaysLiv = rset.getString("Code_Pays_Liv");
			this.clientId = rset.getString("Client_Id");
			this.numTvaClient = rset.getString("Num_TVA_Intra");
			this.zoneUE = (rset.getInt("Zone_UE")==1);
			this.comWebId= rset.getInt("Com_Web_Id");
			this.editionTTC = rset.getInt("Edition_TTC")==1;
			this.directe = rset.getInt("Directe");
			this.typeReglement = rset.getString("Type_R");
			this.modeReglement = rset.getString("Mode_Reg");
			this.tauxTvaPort = rset.getDouble("Taux_TVA_Port");
			this.ecoTaxe = rset.getDouble("Eco_Taxe");
			this.assujetti = rset.getInt("Assujetti_TVA")==1;
			this.TVALiv = rset.getInt("TVA_Liv")==1;
			this.typePays = rset.getString("Type_Pays");
			this.tauxInd = rset.getDouble("Taux_Indicatif");
			this.regimeTVA = rset.getString("Regime_TVA");
				
			if (rset.getString("Nom_Inter").equals("")) {
				this.interlocuteur = "";
			}
			else {
				this.interlocuteur = rset.getString("Civ_Courte_Fact") +" "+ rset.getString("Nom_Inter").toUpperCase() + " " + rset.getString("Prenom_Inter");
			}
			
			if (rset.getString("Nom_Inter_Liv").equals("")) {
				this.interlocuteurLiv = "";
			}
			else {
				this.interlocuteurLiv = rset.getString("Civ_Courte_Liv") +" "+ rset.getString("Nom_Inter_Liv").toUpperCase() + " " + rset.getString("Prenom_Inter_Liv");
			}
			
			
			if (rset.getInt("Imp_Com")==1 && rset.getString("Commentaires").length()>0) {
				this.comFin = "\n"+ rset.getString("Commentaires");
			}
			this.mentions = "\n\n"+ rset.getString("Mentions");
			
			rset.close();
			
			this.exonerationTVA = this.regimeTVA.equals("E") || (this.regimeTVA.equals("G") && !this.zoneUE);
					
			
			// Paramètres dossier
      
			String reqCom = "select Com_Fact, Logo_Adr, Imprimer_Adr_Liv, Imp_LCR from PARAM_DOSSIER where Dossier_Id='"+ dossierId +"'";
			
			rset = stt.executeQuery(reqCom);
			
			rset.next();
			
			logoAdr = rset.getInt("Logo_Adr")==1;
			
			if (rset.getString("Com_Fact").length()>0) {
				this.comFin += "\n\n"+rset.getString("Com_Fact");
			}
			
			this.imprimerAdrLiv = (rset.getInt("Imprimer_Adr_Liv")==1 || !this.codePaysLiv.equals("FR"));
			this.imprimerLCR = (rset.getInt("Imp_LCR")==1  && typeReglement.equals("LCR"));
			
			rset.close();
					
			// modification des coordonnées de l'entreprise facturante si code_pays relié à un site_id dans PARAM_PAYS_WEB
      
			String reqParamPaysWeb = "select * from "+ base +".SITE s,"+ base +".PARAM_PAYS_WEB ppw where ppw.Site_Id=s.Site_Id and ppw.Code_Pays='"+ this.codePays +"'";
			
			rset = stt.executeQuery(reqParamPaysWeb);
			
			if (rset.next()) {
			
				this.raisonSociale = rset.getString("Denomination");
				this.adresse = rset.getString("Adresse_1");
				this.compAdresse = rset.getString("Adresse_2");
				this.adresse3 = rset.getString("Adresse_3");
				this.codePostal = rset.getString("Code_Postal");
				this.ville = rset.getString("Ville");
				this.tel = rset.getString("Tel");
				this.fax = rset.getString("Fax");
				this.email = rset.getString("Email");
				this.tvaIntra = rset.getString("Numero_TVA");
			}
			rset.close();
			

			// ventilation des TVA		
			String reqVTvas = "select a.Taux_TVA, sum(a.Prix * a.Quantite * (1-a.Ristourne/100)) as Base_Montant "
										 + "from "+ base +".ARTICLE_FACTURE a, "+ base +".TAUX_TVA t where Facture_Id="+ factureId +" and a.Code_TVA=t.Code_TVA group by a.Taux_TVA order by a.Taux_TVA";
			
			rset = stt.executeQuery(reqVTvas);
			
			while (rset.next()) {
			
				double tauxTva = rset.getDouble("Taux_TVA");
			
				if (editionTTC) {									
					double montantTVA = (rset.getDouble("Base_Montant") * (1-tRemise/100) * (tauxTva/100) / (tauxTva/100+1));					
					totalTVA += montantTVA;					
					vtvas.add(new VentilTVA(rset.getDouble("Base_Montant") * (1-tRemise/100) - montantTVA, tauxTva));
				} else {
					vtvas.add(new VentilTVA(rset.getDouble("Base_Montant") * (1-tRemise/100), tauxTva));				
					totalTVA += ((tauxTva/100) * rset.getDouble("Base_Montant") * (1-tRemise/100));
				}
			}
			
			rset.close();
			
			// TVA NPR
			String reqTvaNpr = "select t.Taux_NPR, sum(a.Prix * a.Quantite * (1-a.Ristourne/100)) as Base_Montant "
											+ "from "+ base +".ARTICLE_FACTURE a, "+ base +".TAUX_TVA t where Facture_Id="+ factureId +" and t.Code_TVA=a.Code_TVA and t.Taux_NPR>0 group by t.Code_TVA order by t.Taux_NPR";
			
			rset = stt.executeQuery(reqTvaNpr);
			
			while (rset.next()) {
				VentilTVA vtva = new VentilTVA(rset.getDouble("Base_Montant") * (1-tRemise/100), rset.getDouble("Taux_NPR"));
				vtva.setNPR(true);
				vtvasNpr.add(vtva);				
			}
			
			// références affaire, BL et commandes
			this.refAffaire = "";
			this.refCommande = "";
			this.refBL = "";
			
			int nbBons = 0;
			int nbAffaires = 0;
			
			if (directe==0) {	
				String reqRefs = "select a.Type_Fact, a.Num_Entier, a.Affaire_Id, a.Date_Creation from "+ base +".AFFAIRE_FACTURE f,"+ base +".AFFAIRE a where f.Affaire_Id=a.Affaire_Id and f.Facture_Id="+ factureId;

				rset = stt.executeQuery(reqRefs);

				if (rset.next()) {

					this.refAffaire = rset.getString("Num_Entier");
					nbAffaires++;

					String reqBL = "SELECT Num_Entier, Ref_Com from "+ base +".BON_LIVRAISON bl, "+base+".COM_CLIENT cc "+
					               "WHERE bl.Commande_Id=cc.Com_Id AND bl.Facture_Id="+ factureId;
					
					ResultSet rset3 = stt3.executeQuery(reqBL);

					if (rset3.next()) {
						this.refBL = rset3.getString("Num_Entier");
						this.refCommande = rset3.getString("Ref_Com");
						nbBons += 1;

						while (rset3.next()) {
							this.refBL += " - "+ rset3.getString("Num_Entier");
							if (!this.refCommande.matches(".*" + rset3.getString("Ref_Com") + ".*"))
								this.refCommande +=  " - " + rset3.getString("Ref_Com");
							nbBons += 1;
						}
					}

					rset3.close();
				} else if (comWebId>0) {
				
					String reqCommande = "select Com_Site_Id from "+ base +".COMMANDE_WEB where Com_Web_Id="+ comWebId;
					
					ResultSet rset2 = stt2.executeQuery(reqCommande);

					rset2.next();

					this.refCommande = rset2.getString("Com_Site_Id");

					rset2.close();
					
					String reqBLS = "select Num_Entier from "+ base +".BON_LIVRAISON where Etat='V' and Com_Web_Id="+ comWebId;
					
					rset = stt.executeQuery(reqBLS);
					
					if (rset.next()) {
						this.refBL = rset.getString("Num_Entier");
							
						while (rset.next()) {
							this.refBL += " - "+ rset.getString("Num_Entier");
						}
					}					
				}

				rset.close();			
			}
			
			
			// nombre d'exemplaires à sortir et données fiche client
		    if (clientId.equals("")) {
		    	this.comFin += "\n\n";
		        this.nbex = 1;
				this.codeFournisseur = "";
		    } else {

			  String reqEx = "select Nb_Ex, Com_Fact, Code_Fournisseur from "+ base +".FICHE_CLIENT where Client_Id='"+ clientId +"'";
			  rset = stt.executeQuery(reqEx);
			
			  if (rset.next()) {
			    if (rset.getString("Com_Fact").length()>0) {
				    this.comFin += "\n\n"+rset.getString("Com_Fact");
			    }
			    this.nbex = rset.getInt("Nb_Ex") + 1;
				this.codeFournisseur = rset.getString("Code_Fournisseur");
			  }	else {
				this.comFin += "\n\n";
				this.nbex = 1;
				this.codeFournisseur = "";
			  }			
			  rset.close();
		    }					
      
		    // selection des articles appartenant à la facture
			String reqArticles = "select * from "+ base +".ARTICLE_FACTURE where Facture_Id="+ factureId;
			rset = stt.executeQuery(reqArticles);
			
			String reqInfosSup = "select Couleur, Taille, Descrip_1, Descrip_2, Imp_Nom_Facture, Imp_Desc1_Facture, Imp_Desc2_Facture "
												 + "from "+ base +".FICHE_ARTICLE where Article_Id=?";
			PreparedStatement psInfosSup = con.prepareStatement(reqInfosSup);
			
			String reqNomenclature = "select n.ArticleComp_Id, n.Quantite, f.Designation "
														 + "from "+ base +".FICHE_ARTICLE f,"+ base +".COMPOSANT_ARTICLE n where f.Article_Id=n.ArticleComp_Id and n.Article_Id=?";
			PreparedStatement psNomenclature = con.prepareStatement(reqNomenclature);
			
			String reqRefBL = "SELECT cc.Ref_Com, b.Num_Entier, b.Date_Liv, ab.Quantite "+
            				  "FROM "+base+".BON_LIVRAISON b, "+base+".ARTICLE_BON_LIVRAISON ab, "+base+".COM_CLIENT cc " +
			                  "WHERE b.Bon_Id=ab.Bon_Id AND b.Commande_Id=cc.Com_Id AND b.Facture_Id="+ factureId +" AND ab.Ref_Ligne=?";
			PreparedStatement psRefBL = con.prepareStatement(reqRefBL);
			
			TreeMap<String,ArrayList<LigneFacture>> mapBL =  new TreeMap<String,ArrayList<LigneFacture>>();
			
			sansarticles = false;

			while (rset.next()) {
				
				String commentaireAvant = rset.getString("Commentaire_Avant");				
				
				if (commentaireAvant.length()>0) {
				
					String tabCom[] = commentaireAvant.split("\n");
					
					for (int i=0; i<tabCom.length; i++) {
					
						LigneFacture lc = new LigneFacture();
						lc.setType('C');
						lc.setDesignation(tabCom[i]);
						lc.setSousTotal(montantBase);
						lignes.add(lc);
					}
				}
				
				
				LigneFacture lf = new LigneFacture();
				ArrayList<LigneFacture> lignesArticle = new ArrayList<LigneFacture>(10);
				
				String libelle = rset.getString("Libelle");
				
				if (libelle.length()>0) {
					libelle = "\n"+ libelle;
				}			
				
				lf.setReference(rset.getString("Reference"));
				lf.setDesignation(rset.getString("Designation") + libelle);
				lf.setQuantite(rset.getDouble("Quantite"));
				lf.setPrixUnitaire(rset.getDouble("Prix"));
				lf.setRistourne(rset.getDouble("Ristourne"));
				lf.setUnite(rset.getString("Unite"));
				lf.setNumLot(rset.getString("Num_Lot"));
				lf.setNbPieces(rset.getInt("Nb_Pieces"));
				lf.setDatePeremption(rset.getLong("Date_Peremption"));
				lf.setType('A');				
				lf.setPrixIndicatif(rset.getDouble("Prix")*tauxInd);
				
				if (lf.ristourne!=0) rist = true;
				if (!lf.unite.equals("U")) unite = true;
				if (!lf.num_lot.equals("")) numLot = true;
				if (lf.nb_pieces != 0) nbPieces = true;
				if (lf.date_peremption != 0) datePeremption = true;
				
				lf.setMontant(lf.prixUnitaire * lf.quantite * (1-lf.ristourne/100));
				
				montantBase += lf.montant;
				
				lf.setSousTotal(montantBase);
				
				if (directe == 0 && nbAffaires > 1) {
					ResultSet rset2;
					
					// ajout des références affaire et commande
					String reqRefCom = "select a.Num_Entier, cc.Ref_Com from "+ base +".AFFAIRE a, "+ base +".COM_CLIENT cc, "+ base +".ARTICLE_FACTURE af, "+ base +".ARTICLE_COM_CLIENT acc ";
					reqRefCom += "where acc.Ligne_Id=af.Ref_Ligne and cc.Com_Id=acc.Com_Id and cc.Affaire_Id=a.Affaire_Id and af.Facture_Id="+ factureId +" and af.Ref_Ligne="+ rset.getString("Ref_Ligne");
					
					rset2 = stt2.executeQuery(reqRefCom);
					
					if (rset2.next()) {
						lf.setRefBL("R\u00E9f. Affaire : "+ rset2.getString("Num_Entier") +" / R\u00E9f. Com : "+ rset2.getString("Ref_Com") +"\n");
					}
					rset2.close();
				}
				
				lignesArticle.add(lf);				
				
				String commentaireApres = rset.getString("Commentaire");				
				
				if (commentaireApres.length()>0) {
				
					String tabCom[] = commentaireApres.split("\n");
					
					for (int i = 0; i < tabCom.length; i++) {
					
						LigneFacture lc = new LigneFacture();
						lc.setType('C');
						lc.setDesignation(tabCom[i]);
						lc.setSousTotal(montantBase);
						lignesArticle.add(lc);
					}
				}
				
				if (rset.getString("Type_Ligne").equalsIgnoreCase("S")) {					
				
					// récupération infos supplémentaires si existent
					psInfosSup.setString(1, lf.reference);
					ResultSet rsIS = psInfosSup.executeQuery();

					if (rsIS.next()) {

						lf.setCouleur(rsIS.getString("Couleur"));
						lf.setTaille(rsIS.getString("Taille"));

						if (rsIS.getString("Couleur").length()>0) {
							coul = true;
						}

						if (rsIS.getString("Taille").length()>0) {
							tail = true;
						}
						
						if (rsIS.getInt("Imp_Desc1_Facture")==1) {
							lf.setDetail_1(StringUtils.removeHtmlTags(rsIS.getString("Descrip_1")));
						}
						if (rsIS.getInt("Imp_Desc2_Facture")==1) {
							lf.setDetail_2(StringUtils.removeHtmlTags(rsIS.getString("Descrip_2")));
						}

						// gestion de la nomenclature
						if (rsIS.getInt("Imp_Nom_Facture")==1)	{

							psNomenclature.setString(1, lf.reference);
							ResultSet rsN = psNomenclature.executeQuery();

							while (rsN.next()) {
								LigneFacture ln = new LigneFacture();

								ln.setReference(rsN.getString("ArticleComp_Id"));
								ln.setDesignation(rsN.getString("Designation"));
								ln.setQuantite(rsN.getDouble("Quantite"));
								ln.setType('N');
								ln.setSousTotal(montantBase);			

								lignesArticle.add(ln);
							}
							rsN.close();
						}
						
					}
					rsIS.close();								
				}
				
				if (directe == 0 && nbBons > 1) { // On regroupe par bon
					// ajout des références BL
					psRefBL.setInt(1, rset.getInt("Ref_Ligne"));
					ResultSet rsRefBL = psRefBL.executeQuery();
					while (rsRefBL.next()) {
						// La première entrée de lignesArticle est pour l'article
						// dont on écrase la quantité.
						// Mais on doit faire la mise à jour du montant dans la ligne
						// On clone le tableau de lignes de facture pour ne pas les écraser à chaque itération
						ArrayList<LigneFacture> lignesArticleBon = new ArrayList<LigneFacture >(lignesArticle.size());
						Iterator<LigneFacture> it = lignesArticle.iterator();
						while (it.hasNext())
							lignesArticleBon.add((LigneFacture)it.next().clone());
						
						LigneFacture lfbon = lignesArticleBon.get(0);
						// On ne touche pas au montants totaux qui sont fixés lors de chaque
						// itération sur les articles de la facture
						lfbon.setQuantite(rsRefBL.getDouble("Quantite"));
						lfbon.setMontant(lfbon.prixUnitaire * lfbon.quantite * (1-lfbon.ristourne/100));
						
						String numEntier = rsRefBL.getString("Num_Entier");
						if (!mapBL.containsKey(numEntier)) {
							LigneFacture lb = new LigneFacture();
							lb.setType('B');
							String designation = "BL N\u00B0 " + numEntier;
							designation +=  " du " + formatDate.format(new Date(rsRefBL.getLong("Date_Liv")));
							designation += " [commande : " + rsRefBL.getString("Ref_Com") + "]";
							lb.setDesignation(designation);
							ArrayList<LigneFacture>ligneBL = new ArrayList<LigneFacture>(11);
							ligneBL.add(lb);
							ligneBL.addAll(lignesArticleBon);
							mapBL.put(numEntier, ligneBL);
						} else
							mapBL.get(numEntier).addAll(lignesArticleBon);
					}
				} else
					lignes.addAll(lignesArticle);
			}
			
			if (mapBL.size() > 0) { // on a des bons 
				Iterator<ArrayList<LigneFacture>> it = mapBL.values().iterator();
				while (it.hasNext()) {
					lignes.addAll(it.next());
					LigneFacture lt =  new LigneFacture();
					lt.setType('S'); // séparateur de bons
					lignes.add(lt);
				}
			}
			
			if (getNbLignes()==0 && fraisPort>0) {
				sansarticles = true;
				LigneFacture lf = new LigneFacture();
		        					
				lf.setReference("");
				lf.setDesignation("Frais De Livraison");
				lf.setQuantite(1);
				lf.setPrixUnitaire(fraisPort);
				lf.setRistourne(0);
				lf.setType('A');
				if (lf.ristourne!=0) rist = true;
				if (!lf.unite.equals("U")) unite = true;
				if (!lf.num_lot.equals("")) numLot = true;
				if (lf.nb_pieces != 0) nbPieces = true;
				if (lf.date_peremption != 0) datePeremption = true;
				
				lf.setMontant(lf.prixUnitaire * lf.quantite * (1-lf.ristourne/100));
				
				montantBase += lf.montant;
				
				lf.setSousTotal(montantBase);
				lignes.add(lf);
				
			}
			
			double frais = ((sansarticles)?0:fraisPort);
			mRemise = montantBase * tRemise/100;
			totalBase = montantBase + frais - mRemise;
			
						
			if (editionTTC) {
				totalTVA = totalTVA + fraisPort * (tauxTvaPort/100) / (tauxTvaPort/100+1);
				fraisPort = fraisPort / (1 + tauxTvaPort/100); // on a besoin de fraisPort HT
			} else {
				totalTVA = totalTVA + fraisPort * tauxTvaPort/100;
			}

			if (tauxTvaPort > 0 && chercheTauxVentile(fraisPort, tauxTvaPort) == null)
				vtvas.add(new VentilTVA(fraisPort, tauxTvaPort));

			vtvas.addAll(vtvasNpr);
			
			montantTTC = (editionTTC?totalBase:totalBase + totalTVA);
			mEscompte = r.round(montantTTC * tEscompte/100);
			netAPayer = montantTTC - mEscompte - acompte;
			
			
			montantBaseInd=montantBase*tauxInd;
			mRemiseInd=mRemise*tauxInd;
			fraisPortInd=fraisPort*tauxInd;
			totalBaseInd=totalBase*tauxInd;
			totalTVAInd=totalTVA*tauxInd;
			montantTTCInd=montantTTC*tauxInd;
			netAPayerInd=netAPayer*tauxInd;
			mEscompteInd=mEscompte*tauxInd;
			acompteInd=acompte*tauxInd;
		
			rset.close();
			stt.close();			
			s.closeConnection(con, null);
			
		} catch(Exception e) {
			e.printStackTrace();
		}	
	}
	
	
	
	public double calcSousTotal(int derniereLigne) {
	
		if (derniereLigne<0) 
			return 0;
		return lignes.get(derniereLigne).sousTotal;		
	}	
	
	
	public int getNbLignes() {
		return lignes.size();
	}
	
	
	public int getNbEx() {	
		return nbex;
	}
	
	
	public String formatSiren(String siret) {
		if (siret.length()>=9) {
			return siret.substring(0,3) +" "+ siret.substring(3,6) +" "+ siret.substring(6,9);
		}
		return siret;
	}
	
	/**
	 * Ventile une tva dans le tableau si le taux est déjà enregistré
	 * 
	 * @param montant	le montant HT de base
	 * @param taux		le taux de tva à rechercher
	 * @return l'objet VentilTVA trouvé ou null si aucun n'est trouvé
	 */
	private VentilTVA chercheTauxVentile(double montant, double taux) {
		VentilTVA v;
		for (int i = 0; i < vtvas.size(); i++) {
			v = vtvas.get(i);
			if (v.tauxTVA == taux) {
				v.ajouterBase(montant);
				return v;
			}
		}
		return null;
	}
	
	
} // fin DataFacture
